---------------------------------------------------------------------------------
-- Navision Employees Table
---------------------------------------------------------------------------------

-- This script should be run AFTER importing users from Secure Access
-- Again, the Email ("MT Email") is used as the primary key to identify
-- a user.

drop function nav_import_employee();
create or replace function nav_import_employee ()
returns integer as $BODY$
DECLARE
        row		RECORD;

	v_loginname		varchar;
	v_disabled_p		integer;
	v_count			integer;
	v_user_id		integer;
	v_user_login		varchar;
	v_email			varchar;
	v_email_domain		varchar;
	v_email_name		varchar;
	v_supervisor_id		integer;
	v_exists_p		integer;
	v_group_id		integer;
BEGIN
    FOR row IN
        select	*,
		trim(lower(substring("MT Email" from '^([^@]+)'))) as windows_login
	from	nav."Employee"
	order by "Personal ID"
    LOOP

	-- get and massage the email.
	v_email := lower(row."MT Email");
	v_email_domain := substring(v_email from '@(.*)');
	v_email_name := substring(v_email from '(.*)@');
	v_email := v_email_name || '@' || v_email_domain;

	IF row."First Name" is null OR row."First Name" = '' THEN 
	   	RAISE NOTICE 'Skipping User (empty first_name): % % (%)', row."First Name", row."Surname", v_email;
		continue; 
	END IF;
	IF row."Surname" is null OR row."Surname" = '' THEN 
	   	RAISE NOTICE 'Skipping User (empty last_name): % % (%)', row."First Name", row."Surname", v_email;
		continue; 
	END IF;
	IF row."MT Email" is null OR row."MT Email" = '' or v_email is null or v_email = '' THEN 
	   	RAISE NOTICE 'Skipping User (empty email): % % (%)', row."First Name", row."Surname", v_email;
		continue; 
	END IF;

	-- Check if the user already exists.
	v_user_id := null;
	IF v_user_id is null THEN 
		select	min(person_id) into v_user_id from persons
		where	nav_windows_login = row.windows_login;
	END IF;
	IF v_user_id is null THEN 
		select	min(party_id) into v_user_id from parties
		where	email = v_email;
	END IF;
	IF v_user_id is null THEN 
	   	RAISE WARNING 'Did not find user email: % % (%)', row."First Name", row."Surname", v_email;
		continue; 
	END IF;

	RAISE NOTICE 'Processing User: First=%, Last=%, Email=%, OID=% => %', row."First Name", row."Surname", v_email, row."Personal ID", v_user_id;

	-- Get the supervisor and make sure the user is not his own supervisor
	select lower(person_id) into v_supervisor_id from persons
	where nav_windows_login = trim(lower(row."Superior"));
	IF v_supervisor_id = v_user_id THEN v_supervisor_id := NULL; END IF;

	IF row."Superior" is not null and row."Superior" != '' and v_supervisor_id is null THEN
		RAISE WARNING 'Did not find superior % for user %', row."Superior", row."MT Email";
	END IF;

	update persons set
			nav_employee_nr = row."Personal ID"
	where person_id = v_user_id;

	update im_employees set
			supervisor_id = v_supervisor_id
	where employee_id = v_user_id;


	-- Make the new user an Employee
	select group_id into v_group_id from groups
	where group_name = 'Employees';

	IF v_exists_p = 0 THEN
		RAISE NOTICE 'Adding: user=%, group=%', v_user_id, v_group_id;
		PERFORM membership_rel__new(
			null,			-- rel_id, default null  
			'membership_rel',	-- object_type, default membership_rel
			v_group_id,		-- object_id_one
			v_user_id,		-- object_id_two
			'approved',		-- new__member_state, default approved
			null,			-- creation_user, default null
			'0.0.0.0'		-- creation_ip, default null
		);
	ELSE
		RAISE NOTICE 'User % already member of group Employees', v_user_id;
	END IF;



    END LOOP;
    RETURN 0;
END;$BODY$ language 'plpgsql';
select nav_import_employee ();



